"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformTokens = transformTokens;

var t = _interopRequireWildcard(require("@babel/types"));

var _defaultVariant = require("../lib/utils/defaultVariant");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// Packages
function transformTokens(tokens, stylesheetId, addStylesheet) {
  const isDefaultOnly = tokens.every(token => (0, _defaultVariant.isDefaultVariant)(token.variant));

  if (tokens.length === 0) {
    return {
      isDefaultOnly,
      ast: t.objectExpression([])
    };
  }

  if (isDefaultOnly) {
    // We know for sure it's a single element
    const {
      variant,
      style,
      ...props
    } = tokens[0];
    const id = addStylesheet({
      variant,
      style
    });
    const outStyle = t.valueToNode(props);
    outStyle.properties.push(t.objectProperty(t.identifier('style'), t.arrayExpression([t.memberExpression(stylesheetId, id)])));
    return {
      isDefaultOnly,
      ast: outStyle
    };
  }

  const ast = t.arrayExpression(tokens.map(({
    variant,
    style,
    ...props
  }) => {
    const id = addStylesheet({
      variant,
      style
    });
    const outStyle = t.valueToNode({
      variant,
      ...props
    });
    outStyle.properties.push(t.objectProperty(t.identifier('style'), t.memberExpression(stylesheetId, id)));
    return outStyle;
  }));
  return {
    isDefaultOnly,
    ast
  };
}
//# sourceMappingURL=transform-tokens.js.map