"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveTokens = void 0;

var _dequal = require("dequal");

var _theme = _interopRequireDefault(require("./theme"));

var _style = _interopRequireDefault(require("./style"));

var _reduce = require("./utils/reduce");

var _props = _interopRequireDefault(require("./style/props"));

var _extractTokenInfo = require("./utils/extractTokenInfo");

var _getOrderedTokens = require("./utils/getOrderedTokens");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Packages
// Ours
const themeStyles = (0, _style.default)(_theme.default);
const themeProps = (0, _props.default)(_theme.default);

const resolveToken = token => {
  const {
    styleId,
    variant
  } = (0, _extractTokenInfo.extractTokenInfo)(token);
  const style = themeStyles[styleId];
  const props = themeProps[styleId];

  if (!style && !props) {
    throw new Error("Unknown style: '".concat(styleId, "'"));
  }

  return {
    variant,
    style,
    ...props
  };
};

const resolveTokens = tokens => {
  const ordered = (0, _getOrderedTokens.getOrderedTokens)(tokens); // Different order? warn the user

  if (!(0, _dequal.dequal)(ordered, tokens)) {
    throw new Error('Incorrect order. Default variant styles must always come first.');
  }

  return (0, _reduce.reduce)(ordered.map(resolveToken));
};

exports.resolveTokens = resolveTokens;
//# sourceMappingURL=tokens.js.map