"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.select = void 0;

var _variants = require("../variants");

// Ours
const select = (styles, variantFlags) => {
  // Override built-in variants with user provided values but always
  // include default variant styles
  const flags = { ..._variants.builtinVariants,
    ...variantFlags,
    default: true
  }; // Check if a variant is enabled

  const isEnabled = variant => {
    return variant.split(':').map(v => flags[v]).every(Boolean);
  };

  const mergeStyles = (base, override) => {
    if (!base.style) {
      return [override.style];
    }

    if (Array.isArray(base.style)) {
      return [...base.style, override.style];
    }

    return [base.style, override.style];
  };

  return styles.filter(({
    variant
  }) => isEnabled(variant)).map(({
    variant: _,
    ...style
  }) => style).reduce((current, next) => ({ ...current,
    ...next,
    style: mergeStyles(current, next)
  }), {});
};

exports.select = select;
//# sourceMappingURL=select.js.map