"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var t = _interopRequireWildcard(require("@babel/types"));

var _babelPluginMacros = require("babel-plugin-macros");

var _tokens = require("./lib/tokens");

var _evalNode = require("./babel/eval-node");

var _addImport = require("./babel/add-import");

var _injectStyles = require("./babel/inject-styles");

var _transformTokens = require("./babel/transform-tokens");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// Packages
const styledMacro = ({
  references,
  state
}) => {
  var _references$default;

  const program = state.file.path; // A map that holds the key/value pairs of the generated styles.

  const stylesheet = {}; // Variable name to be used later with StyleSheet.create

  const stylesheetId = program.scope.generateUidIdentifier('styles');
  (_references$default = references.default) === null || _references$default === void 0 ? void 0 : _references$default.forEach(refPath => {
    if (!t.isCallExpression(refPath.parent)) {
      return;
    } // Get the call expression


    const callExpr = refPath.parent; // Parse token argument

    const {
      confident,
      value: tokens,
      deopt
    } = (0, _evalNode.evalNode)(refPath.parentPath, callExpr.arguments[0]);

    if (!confident) {
      throw deopt === null || deopt === void 0 ? void 0 : deopt.buildCodeFrameError('could not evaluate style names at compile time');
    }

    if (!Array.isArray(tokens)) {
      throw deopt === null || deopt === void 0 ? void 0 : deopt.buildCodeFrameError('style names must be an array of strings');
    }

    try {
      const styleAST = (0, _transformTokens.transformTokens)((0, _tokens.resolveTokens)(tokens), stylesheetId, ({
        variant,
        style
      }) => {
        const id = program.scope.generateUidIdentifier(variant);
        stylesheet[id.name] = style;
        return id;
      }); // Avoid wrapping styles around select(..) call if we only have
      // defautl variant styles.

      if (styleAST.isDefaultOnly) {
        // => { style: [defaultStyle] , ...props }
        refPath.parentPath.replaceWith(styleAST.ast);
        return;
      } // { style: [...styles] , ...props }


      callExpr.arguments[0] = styleAST.ast; // Import StyleUtils.select
      // => import { select } from 'path/to/util';
      //    select([...], ...)

      callExpr.callee = (0, _addImport.importUtil)(program, 'select');
    } catch (error) {
      throw refPath.buildCodeFrameError(error.message);
    }
  }); // Inject generated styles to the module

  if (Object.keys(stylesheet).length > 0) {
    (0, _injectStyles.injectStyles)(program, stylesheetId, stylesheet);
  }
};

var _default = (0, _babelPluginMacros.createMacro)(styledMacro);

exports.default = _default;
//# sourceMappingURL=macro.js.map