// Packages
import * as t from '@babel/types';
import { createMacro } from 'babel-plugin-macros'; // Ours

import { resolveTokens } from './lib/tokens';
import { evalNode } from './babel/eval-node';
import { importUtil } from './babel/add-import';
import { injectStyles } from './babel/inject-styles';
import { transformTokens } from './babel/transform-tokens';

const styledMacro = ({
  references,
  state
}) => {
  var _references$default;

  const program = state.file.path; // A map that holds the key/value pairs of the generated styles.

  const stylesheet = {}; // Variable name to be used later with StyleSheet.create

  const stylesheetId = program.scope.generateUidIdentifier('styles');
  (_references$default = references.default) === null || _references$default === void 0 ? void 0 : _references$default.forEach(refPath => {
    if (!t.isCallExpression(refPath.parent)) {
      return;
    } // Get the call expression


    const callExpr = refPath.parent; // Parse token argument

    const {
      confident,
      value: tokens,
      deopt
    } = evalNode(refPath.parentPath, callExpr.arguments[0]);

    if (!confident) {
      throw deopt === null || deopt === void 0 ? void 0 : deopt.buildCodeFrameError('could not evaluate style names at compile time');
    }

    if (!Array.isArray(tokens)) {
      throw deopt === null || deopt === void 0 ? void 0 : deopt.buildCodeFrameError('style names must be an array of strings');
    }

    try {
      const styleAST = transformTokens(resolveTokens(tokens), stylesheetId, ({
        variant,
        style
      }) => {
        const id = program.scope.generateUidIdentifier(variant);
        stylesheet[id.name] = style;
        return id;
      }); // Avoid wrapping styles around select(..) call if we only have
      // defautl variant styles.

      if (styleAST.isDefaultOnly) {
        // => { style: [defaultStyle] , ...props }
        refPath.parentPath.replaceWith(styleAST.ast);
        return;
      } // { style: [...styles] , ...props }


      callExpr.arguments[0] = styleAST.ast; // Import StyleUtils.select
      // => import { select } from 'path/to/util';
      //    select([...], ...)

      callExpr.callee = importUtil(program, 'select');
    } catch (error) {
      throw refPath.buildCodeFrameError(error.message);
    }
  }); // Inject generated styles to the module

  if (Object.keys(stylesheet).length > 0) {
    injectStyles(program, stylesheetId, stylesheet);
  }
};

export default createMacro(styledMacro);
//# sourceMappingURL=macro.js.map