import { UnionToIntersection, ValueOf } from 'type-fest';
import { Theme } from './theme';
import createStyles from './style';
import createProps from './style/props';
export declare type TokenInfo = {
    variant: string;
    style: Record<string, any>;
    [props: string]: unknown;
};
export declare type VariantFlags = Record<string, boolean | undefined>;
export declare type Configuration = {
    theme: Theme;
};
declare type Styles = UnionToIntersection<ReturnType<typeof createStyles>>;
declare type Props = UnionToIntersection<ReturnType<typeof createProps>>;
declare type Token = keyof UnionToIntersection<Styles | Props>;
declare type TokenWithVariant = Token | `${string}:${Token}`;
declare type ExtractToken<T extends string> = T extends `${string}:${infer V}` ? V extends Token ? V : ExtractToken<V> : T extends Token ? T : never;
declare type ComputedStyle<T extends Token> = UnionToIntersection<T extends keyof Styles ? {
    style: ValueOf<Styles, T>[];
} : never | T extends keyof Props ? ValueOf<Props, T> : never>;
export declare type Macro = <T extends TokenWithVariant>(tokens: T[], variants?: VariantFlags) => ComputedStyle<ExtractToken<T>>;
export {};
